***********************************************************************
* Copyright 1998,1999,2000,2001,2002,2005,2007,2008,2009,2010         *
* Andreas Stohl, Petra Seibert, A. Frank, Gerhard Wotawa,             *
* Caroline Forster, Sabine Eckhardt, John Burkhart, Harald Sodemann   *
*                                                                     *
* This file is part of FLEXPART.                                      *
*                                                                     *
* FLEXPART is free software: you can redistribute it and/or modify    *
* it under the terms of the GNU General Public License as published by*
* the Free Software Foundation, either version 3 of the License, or   *
* (at your option) any later version.                                 *
*                                                                     *
* FLEXPART is distributed in the hope that it will be useful,         *
* but WITHOUT ANY WARRANTY; without even the implied warranty of      *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
* GNU General Public License for more details.                        *
*                                                                     *
* You should have received a copy of the GNU General Public License   *
* along with FLEXPART.  If not, see <http://www.gnu.org/licenses/>.   *
***********************************************************************

      subroutine assignland()
********************************************************************************
*                                                                              *
*     This routine assigns fractions of the 13 landuse classes to each ECMWF   *
*     grid point.                                                              *
*     The landuse inventory of                                                 *
*                                                                              *
* Belward, A.S., Estes, J.E., and Kline, K.D., 1999,                           * 
* The IGBP-DIS 1-Km Land-Cover Data Set DISCover:                              *     
* A Project Overview: Photogrammetric Engineering and Remote Sensing ,         *   
* v. 65, no. 9, p. 1013-1020                                                   *    
*                                                                              *  
*     if there are no data in the inventory                                    *         
*     the ECMWF land/sea mask is used to distinguish                           *
*     between sea (-> ocean) and land (-> grasslands).                         *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     5 December 1996                                                          *
*     8 February 1999 Additional use of nests, A. Stohl                        *
*    29 December 2006 new landuse inventory, S. Eckhardt                       *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* xlanduse          fractions of numclass landuses for each model grid point   *
* landinvent       landuse inventory (0.3 deg resolution)                      *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer ix,jy,k,l,li,nrefine,iix,jjy,lumaxx,lumaxy,xlon0lu,ylat0lu
      real dxlu
      parameter(lumaxx=1200,lumaxy=600)
      parameter(xlon0lu=-180,ylat0lu=-90)
      parameter(dxlu=0.3)
      real xlon,ylat,sumperc,p,xi,yj
      real xlandusep(lumaxx,lumaxy,numclass)
c      character*2 ck

      do 5 ix=1,lumaxx
        do 5 jy=1,lumaxy
              do 7 k=1,numclass
7               xlandusep(ix,jy,k)=0.
              sumperc=0.
              do 8 li=1,3
8               sumperc=sumperc+landinvent(ix,jy,li+3)
              do 9 li=1,3
                k=landinvent(ix,jy,li)
              if (sumperc.gt.0) then
                p=landinvent(ix,jy,li+3)/sumperc 
              else
                p=0
              endif
c p has values between 0 and 1             
9              xlandusep(ix,jy,k)=p 
5     continue

c      do 13 k=1,11
c      write (ck,'(i2.2)') k
c      open(4,file='xlandusetest'//ck,form='formatted')
c      do 11 ix=1,lumaxx
c11       write (4,*) (xlandusep(ix,jy,k),jy=1,lumaxy)
c11       write (4,*) (landinvent(ix,jy,k),jy=1,lumaxy)
c13     close(4)

c      write (*,*) xlon0,ylat0,xlon0n(1),ylat0n(1),nxmin1,nymin1
c      write (*,*) dx, dy, dxout, dyout, ylat0, xlon0
      nrefine=10 
      do 10 ix=0,nxmin1
        do 10 jy=0,nymin1
          do 20 k=1,numclass
            sumperc=0.
20          xlanduse(ix,jy,k)=0.
            do 30 iix=1, nrefine
              xlon=(ix+(iix-1)/float(nrefine))*dx+xlon0        ! longitude, should be between -180 and 179
              if (xlon.ge.(xlon0lu+lumaxx*dxlu))  then
                   xlon=xlon-lumaxx*dxlu
              endif
              do 30 jjy=1, nrefine
               ylat=(jy+(jjy-1)/float(nrefine))*dy+ylat0       ! and lat. of each gridpoint
               xi=int((xlon-xlon0lu)/dxlu)+1
               yj=int((ylat-ylat0lu)/dxlu)+1
               if (xi.gt.lumaxx) xi=xi-lumaxx
               if (yj.gt.lumaxy) yj=yj-lumaxy
               if (xi.lt.0) then
                  write (*,*) 'problem with landuseinv sampling: ',
     +                           xlon,xlon0lu,ix,iix,xlon0,dx,nxmax
                  stop
               endif
               do 33 k=1,numclass
                  xlanduse(ix,jy,k)=
     +                xlanduse(ix,jy,k)+xlandusep(int(xi),int(yj),k)
33                sumperc=sumperc+xlanduse(ix,jy,k)  ! just for the check if landuseinv. is available
30            continue
              if (sumperc.gt.0) then                       ! detailed landuse available
              sumperc=0.
              do 50 k=1,numclass
                  xlanduse(ix,jy,k)=
     +                xlanduse(ix,jy,k)/float(nrefine*nrefine)
50                 sumperc=sumperc+xlanduse(ix,jy,k)
ccc the sum of all categories should be 1 ... 100 percent ... in order to get vdep right!
              if (sumperc.lt.1-1E-5) then
                do 52 k=1,numclass
52                  xlanduse(ix,jy,k)=
     +                xlanduse(ix,jy,k)/sumperc
              endif
              else                                
                if (lsm(ix,jy).lt.0.1) then           ! over sea  -> ocean
                  xlanduse(ix,jy,3)=1.
                else                                  ! over land -> rangeland
                  xlanduse(ix,jy,7)=1.
                endif
              endif


10        continue

************************************
* for test: write out xlanduse 

c      open(4,file='landusetest',form='formatted')
c      do 56 k=1,13
c      do 55 ix=0,nxmin1
c55    write (4,*) (xlanduse(ix,jy,k),jy=0,nymin1)
c56    continue
c      close(4)
c      write (*,*) 'landuse written'
c     stop
c       open(4,file='landseatest'//ck,form='formatted')
c      do 57 ix=0,nxmin1
c57       write (4,*) (lsm(ix,jy),jy=0,nymin1)
c       write (*,*) 'landseamask written'

*****************************************
C Same as above, but for the nested grids
*****************************************

*************** TEST ********************       
c      dyn(1)=dyn(1)/40
c      dxn(1)=dxn(1)/40
c      xlon0n(1)=1
c      ylat0n(1)=50
*************** TEST ********************

      do 60 l=1,numbnests
        do 60 ix=0,nxn(l)-1
          do 60 jy=0,nyn(l)-1
            do 70 k=1,numclass
              sumperc=0.
70            xlandusen(ix,jy,k,l)=0.
              do 80 iix=1, nrefine
               xlon=(ix+(iix-1)/float(nrefine))*dxn(l)+xlon0n(l)     
               do 80 jjy=1, nrefine
                 ylat=(jy+(jjy-1)/float(nrefine))*dyn(l)+ylat0n(l) 
                 xi=int((xlon-xlon0lu)/dxlu)+1
                 yj=int((ylat-ylat0lu)/dxlu)+1
                 if (xi.gt.lumaxx) xi=xi-lumaxx
                 if (yj.gt.lumaxy) yj=yj-lumaxy
                 do 90 k=1,numclass
                    xlandusen(ix,jy,k,l)=xlandusen(ix,jy,k,l)+
     +              xlandusep(int(xi),int(yj),k)
90                  sumperc=sumperc+xlandusen(ix,jy,k,l)
80            continue
              if (sumperc.gt.0) then                     ! detailed landuse available
              sumperc=0.
                do 100 k=1,numclass
                   xlandusen(ix,jy,k,l)=
     +                  xlandusen(ix,jy,k,l)/float(nrefine*nrefine)
100                sumperc=sumperc+xlandusen(ix,jy,k,l)
ccc the sum of all categories should be 1 ... 100 percent ... in order to get vdep right!
                if (sumperc.lt.1-1E-5) then
                  do 102 k=1,numclass
102                xlandusen(ix,jy,k,l)=xlandusen(ix,jy,k,l)/sumperc
                endif
              else                                    ! check land/sea mask
                if (lsmn(ix,jy,l).lt.0.1) then   ! over sea  -> ocean
                  xlandusen(ix,jy,3,l)=1.
                else                        ! over land -> grasslands
                  xlandusen(ix,jy,7,l)=1.
                endif
              endif
60          continue

************************************
* for test: write out xlanduse 

c      do 66 k=1,11
c      write (ck,'(i2.2)') k
c      open(4,file='nlandusetest'//ck,form='formatted')
c      do 65 ix=0,nxn(1)-1
c65       write (4,*) (xlandusen(ix,jy,k,1),jy=0,nyn(1)-1)
c66      close(4)

c      write (*,*) 'landuse nested written'


      end
