***********************************************************************
* Copyright 1998,1999,2000,2001,2002,2005,2007,2008,2009,2010         *
* Andreas Stohl, Petra Seibert, A. Frank, Gerhard Wotawa,             *
* Caroline Forster, Sabine Eckhardt, John Burkhart, Harald Sodemann   *
*                                                                     *
* This file is part of FLEXPART.                                      *
*                                                                     *
* FLEXPART is free software: you can redistribute it and/or modify    *
* it under the terms of the GNU General Public License as published by*
* the Free Software Foundation, either version 3 of the License, or   *
* (at your option) any later version.                                 *
*                                                                     *
* FLEXPART is distributed in the hope that it will be useful,         *
* but WITHOUT ANY WARRANTY; without even the implied warranty of      *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
* GNU General Public License for more details.                        *
*                                                                     *
* You should have received a copy of the GNU General Public License   *
* along with FLEXPART.  If not, see <http://www.gnu.org/licenses/>.   *
***********************************************************************

      subroutine calcfluxes(nage,jpart,xold,yold,zold)
C                            i     i    i    i    i  
********************************************************************************
*                                                                              *
*     Calculation of the gross fluxes across horizontal, eastward and          *
*     northward facing surfaces. The routine calculates the mass flux          *
*     due to the motion of only one particle. The fluxes of subsequent calls   *
*     to this subroutine are accumulated until the next output is due.         *
*     Upon output, flux fields are re-set to zero in subroutine fluxoutput.f.  *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     04 April 2000                                                            *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
*                                                                              *
* nage                  Age class of the particle considered                   *
* jpart                 Index of the particle considered                       *
* xold,yold,zold        "Memorized" old positions of the particle              *
*                                                                              *
********************************************************************************

      use flux_mod
      use outg_mod

      include 'includepar'
      include 'includecom'

      integer jpart,nage,ixave,jyave,kz,kzave,kp
      integer k,k1,k2,ix,ix1,ix2,ixs,jy,jy1,jy2
      real xold,yold,zold,xmean,ymean


C Determine average positions
*****************************

      if (ioutputforeachrelease.eq.1) then
         kp=npoint(jpart)
      else
         kp=1
      endif
      
      xmean=(xold+xtra1(jpart))/2.
      ymean=(yold+ytra1(jpart))/2.

      ixave=int((xmean*dx+xoutshift)/dxout)
      jyave=int((ymean*dy+youtshift)/dyout)
      do 15 kz=1,numzgrid                ! determine height of cell
        if (outheight(kz).gt.ztra1(jpart)) goto 16
15      continue
16      kzave=kz


C Determine vertical fluxes
***************************

      if ((ixave.ge.0).and.(jyave.ge.0).and.(ixave.le.numxgrid-1).and.
     +(jyave.le.numygrid-1)) then
        do 10 kz=1,numzgrid                ! determine height of cell
          if (outheighthalf(kz).gt.zold) goto 11
10        continue
11      k1=min(numzgrid,kz)
        do 20 kz=1,numzgrid                ! determine height of cell
          if (outheighthalf(kz).gt.ztra1(jpart)) goto 21
20        continue
21      k2=min(numzgrid,kz)

        do 40 k=1,nspec
          do 41 kz=k1,k2-1
41          flux(5,ixave,jyave,kz,k,kp,nage)=
     +          flux(5,ixave,jyave,kz,k,kp,nage)+
     +          xmass1(jpart,k)
          do 40 kz=k2,k1-1
40          flux(6,ixave,jyave,kz,k,kp,nage)=
     +          flux(6,ixave,jyave,kz,k,kp,nage)+
     +          xmass1(jpart,k)
      endif


C Determine west-east fluxes (fluxw) and east-west fluxes (fluxe)
*****************************************************************

      if ((kzave.le.numzgrid).and.(jyave.ge.0).and.
     +(jyave.le.numygrid-1)) then

C 1) Particle does not cross domain boundary

        if (abs(xold-xtra1(jpart)).lt.float(nx)/2.) then
          ix1=int((xold*dx+xoutshift)/dxout+0.5)
          ix2=int((xtra1(jpart)*dx+xoutshift)/dxout+0.5)
          do 50 k=1,nspec
            do 51 ix=ix1,ix2-1
              if ((ix.ge.0).and.(ix.le.numxgrid-1)) then
                flux(1,ix,jyave,kzave,k,kp,nage)=
     +              flux(1,ix,jyave,kzave,k,kp,nage)
     +              +xmass1(jpart,k)
              endif
51            continue
            do 50 ix=ix2,ix1-1
              if ((ix.ge.0).and.(ix.le.numxgrid-1)) then
                flux(2,ix,jyave,kzave,k,kp,nage)=
     +          flux(2,ix,jyave,kzave,k,kp,nage)
     +             +xmass1(jpart,k)
              endif
50            continue

C 2) Particle crosses domain boundary: use cyclic boundary condition
C    and attribute flux to easternmost grid row only (approximation valid
C    for relatively slow motions compared to output grid cell size)

        else
          ixs=int(((float(nxmin1)-1.e5)*dx+xoutshift)/dxout)
          if ((ixs.ge.0).and.(ixs.le.numxgrid-1)) then
            if (xold.gt.xtra1(jpart)) then       ! west-east flux
              do 52 k=1,nspec
52              flux(1,ixs,jyave,kzave,k,kp,nage)=
     +             flux(1,ixs,jyave,kzave,k,kp,nage)
     +             +xmass1(jpart,k)
            else                                 ! east-west flux
              do 53 k=1,nspec
53              flux(2,ixs,jyave,kzave,k,kp,nage)=
     +             flux(2,ixs,jyave,kzave,k,kp,nage)
     +             +xmass1(jpart,k)
            endif
          endif
        endif
      endif


C Determine south-north fluxes (fluxs) and north-south fluxes (fluxn)
*********************************************************************

      if ((kzave.le.numzgrid).and.(ixave.ge.0).and.
     +(ixave.le.numxgrid-1)) then
        jy1=int((yold*dy+youtshift)/dyout+0.5)
        jy2=int((ytra1(jpart)*dy+youtshift)/dyout+0.5)

        do 60 k=1,nspec
          do 61 jy=jy1,jy2-1
            if ((jy.ge.0).and.(jy.le.numygrid-1)) then
              flux(3,ixave,jy,kzave,k,kp,nage)=
     +          flux(3,ixave,jy,kzave,k,kp,nage)
     +          +xmass1(jpart,k)
            endif
61          continue
          do 60 jy=jy2,jy1-1
            if ((jy.ge.0).and.(jy.le.numygrid-1)) then
              flux(4,ixave,jy,kzave,k,kp,nage)=
     +           flux(4,ixave,jy,kzave,k,kp,nage)
     +           +xmass1(jpart,k)
            endif
60          continue
      endif

      end
