***********************************************************************
* Copyright 1998,1999,2000,2001,2002,2005,2007,2008,2009,2010         *
* Andreas Stohl, Petra Seibert, A. Frank, Gerhard Wotawa,             *
* Caroline Forster, Sabine Eckhardt, John Burkhart, Harald Sodemann   *
*                                                                     *
* This file is part of FLEXPART.                                      *
*                                                                     *
* FLEXPART is free software: you can redistribute it and/or modify    *
* it under the terms of the GNU General Public License as published by*
* the Free Software Foundation, either version 3 of the License, or   *
* (at your option) any later version.                                 *
*                                                                     *
* FLEXPART is distributed in the hope that it will be useful,         *
* but WITHOUT ANY WARRANTY; without even the implied warranty of      *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
* GNU General Public License for more details.                        *
*                                                                     *
* You should have received a copy of the GNU General Public License   *
* along with FLEXPART.  If not, see <http://www.gnu.org/licenses/>.   *
***********************************************************************

      subroutine concoutput(itime,outnum,gridtotalunc,wetgridtotalunc,
     +drygridtotalunc)
C                             i     i          o             o
C            o
********************************************************************************
*                                                                              *
*     Output of the concentration grid and the receptor concentrations.        *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     24 May 1995                                                              *
*                                                                              *
*     13 April 1999, Major update: if output size is smaller, dump output      *
*                    in sparse matrix format; additional output of uncertainty *
*                                                                              *
*     05 April 2000, Major update: output of age classes; output for backward  *
*                    runs is time spent in grid cell times total mass of       *
*                    species.                                                  *
*                                                                              *
*     17 February 2002, Appropriate dimensions for backward and forward runs   *
*                       are now specified in file includepar                   *
*                                                                              *
*     June 2006, write grid in sparse matrix with a single write command       * 
*                in order to save disk space                                   *      
*                                                                              *
*     2008 new sparse matrix format      
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* outnum          number of samples                                            *
* ncells          number of cells with non-zero concentrations                 *
* sparse          .true. if in sparse matrix format, else .false.              *
* tot_mu          1 for forward, initial mass mixing ration for backw. runs    *
*                                                                              *
********************************************************************************

      use unc_mod
      use point_mod
      use outg_mod

      include 'includepar'
      include 'includecom'

      double precision jul
      integer itime,i,ix,jy,kz,ks,kp,l,iix,jjy,kzz,nage,jjjjmmdd,ihmmss
      integer sp_count_i,sp_count_r
      real sp_fact
      real outnum,weightair,densityoutrecept(maxreceptor),xl,yl

c     real densityoutgrid(0:numxgrid-1,0:numygrid-1,numzgrid),
c    +grid(0:numxgrid-1,0:numygrid-1,numzgrid,maxspec,maxpointspec_act,
c    +    maxageclass)
c     real wetgrid(0:numxgrid-1,0:numygrid-1,maxspec,maxpointspec_act,
c    +       maxageclass)
c     real drygrid(0:numxgrid-1,0:numygrid-1,maxspec,
c    +       maxpointspec_act,maxageclass)
c     real gridsigma(0:numxgrid-1,0:numygrid-1,numzgrid,maxspec,
c    +       maxpointspec_act,maxageclass),
c    +     drygridsigma(0:numxgrid-1,0:numygrid-1,maxspec,
c    +     maxpointspec_act,maxageclass),
c    +     wetgridsigma(0:numxgrid-1,0:numygrid-1,maxspec,
c    +     maxpointspec_act,maxageclass)
c     real factor(0:numxgrid-1,0:numygrid-1,numzgrid)
c     real sparse_dump_r(numxgrid*numygrid*numzgrid)
c     integer sparse_dump_i(numxgrid*numygrid*numzgrid)

c     real sparse_dump_u(numxgrid*numygrid*numzgrid)
      real auxgrid(nclassunc),gridtotal,gridsigmatotal,gridtotalunc
      real wetgridtotal,wetgridsigmatotal,wetgridtotalunc
      real drygridtotal,drygridsigmatotal,drygridtotalunc
      real halfheight,dz,dz1,dz2,tot_mu(maxspec,maxpointspec_act)
      real smallnum
      save smallnum
      data smallnum /1.e-38/ ! should be smallest number that can be handled
      parameter(weightair=28.97)
      logical sp_zer
      character adate*8,atime*6
      character*3 anspec


C Determine current calendar date, needed for the file name
***********************************************************

      jul=bdate+dble(float(itime))/86400.
      call caldate(jul,jjjjmmdd,ihmmss)
      write(adate,'(i8.8)') jjjjmmdd
      write(atime,'(i6.6)') ihmmss
      write(unitdates,'(a)') adate//atime


C For forward simulations, output fields have dimension MAXSPEC,
C for backward simulations, output fields have dimension MAXPOINT.
C Thus, make loops either about nspec, or about numpoint
******************************************************************

       
      if (ldirect.eq.1) then
        do 7 ks=1,nspec
          do 7 kp=1,maxpointspec_act
7           tot_mu(ks,kp)=1
      else
        do 10 ks=1,nspec
          do 8 kp=1,maxpointspec_act
8           tot_mu(ks,kp)=xmass(kp,ks)
10        continue
      endif


********************************************************************
C Compute air density: sufficiently accurate to take it
C from coarse grid at some time
C Determine center altitude of output layer, and interpolate density
C data to that altitude
********************************************************************

      do 44 kz=1,numzgrid
        if (kz.eq.1) then
          halfheight=outheight(1)/2.
        else
          halfheight=(outheight(kz)+outheight(kz-1))/2.
        endif
        do 45 kzz=2,nz
          if ((height(kzz-1).lt.halfheight).and.
     +    (height(kzz).gt.halfheight)) goto 46
45        continue
46      kzz=max(min(kzz,nz),2)
        dz1=halfheight-height(kzz-1)
        dz2=height(kzz)-halfheight
        dz=dz1+dz2
        do 44 jy=0,numygrid-1
          do 44 ix=0,numxgrid-1
            xl=outlon0+float(ix)*dxout
            yl=outlat0+float(jy)*dyout
            xl=(xl-xlon0)/dx
            yl=(yl-ylat0)/dx
            iix=max(min(nint(xl),nxmin1),0)
            jjy=max(min(nint(yl),nymin1),0)
44          densityoutgrid(ix,jy,kz)=(rho(iix,jjy,kzz,2)*dz1+
     +      rho(iix,jjy,kzz-1,2)*dz2)/dz

        do 47 i=1,numreceptor
          xl=xreceptor(i)
          yl=yreceptor(i)
          iix=max(min(nint(xl),nxmin1),0)
          jjy=max(min(nint(yl),nymin1),0)
47        densityoutrecept(i)=rho(iix,jjy,1,2)


C Output is different for forward and backward simulations
        do 27 kz=1,numzgrid
          do 27 jy=0,numygrid-1
            do 27 ix=0,numxgrid-1
              if (ldirect.eq.1) then
                factor3d(ix,jy,kz)=1.e12/volume(ix,jy,kz)/outnum
              else
                factor3d(ix,jy,kz)=float(abs(loutaver))/outnum
              endif
27            continue

**********************************************************************
C Determine the standard deviation of the mean concentration or mixing
C ratio (uncertainty of the output) and the dry and wet deposition
**********************************************************************

      gridtotal=0.
      gridsigmatotal=0.
      gridtotalunc=0.
      wetgridtotal=0.
      wetgridsigmatotal=0.
      wetgridtotalunc=0.
      drygridtotal=0.
      drygridsigmatotal=0.
      drygridtotalunc=0.

      do 2 ks=1,nspec

      write(anspec,'(i3.3)') ks
      if ((iout.eq.1).or.(iout.eq.3).or.(iout.eq.5)) then
        if (ldirect.eq.1) then
          open(unitoutgrid,file=path(2)(1:len(2))//'grid_conc_'//adate//
     +    atime//'_'//anspec,form='unformatted')
        else
          open(unitoutgrid,file=path(2)(1:len(2))//'grid_time_'//adate//
     +    atime//'_'//anspec,form='unformatted')
        endif
         write(unitoutgrid) itime
       endif

      if ((iout.eq.2).or.(iout.eq.3)) then      ! mixing ratio
       open(unitoutgridppt,file=path(2)(1:len(2))//'grid_pptv_'//adate//
     +  atime//'_'//anspec,form='unformatted')

        write(unitoutgridppt) itime
      endif

      do 12 kp=1,maxpointspec_act
      do 12 nage=1,nageclass

        do 120 jy=0,numygrid-1
          do 120 ix=0,numxgrid-1

C WET DEPOSITION
            if ((WETDEP).and.(ldirect.gt.0)) then
                do 122 l=1,nclassunc
122               auxgrid(l)=wetgridunc(ix,jy,ks,kp,l,nage)
                call mean(auxgrid,wetgrid(ix,jy),
     +          wetgridsigma(ix,jy),nclassunc)
C Multiply by number of classes to get total concentration
                wetgrid(ix,jy)=wetgrid(ix,jy)
     +          *nclassunc
                wetgridtotal=wetgridtotal+wetgrid(ix,jy)
C Calculate standard deviation of the mean
                wetgridsigma(ix,jy)=
     +          wetgridsigma(ix,jy)*
     +          sqrt(float(nclassunc))
                wetgridsigmatotal=wetgridsigmatotal+
     +          wetgridsigma(ix,jy)
            endif

C DRY DEPOSITION
            if ((DRYDEP).and.(ldirect.gt.0)) then
                do 123 l=1,nclassunc
123               auxgrid(l)=drygridunc(ix,jy,ks,kp,l,nage)
                call mean(auxgrid,drygrid(ix,jy),
     +          drygridsigma(ix,jy),nclassunc)
C Multiply by number of classes to get total concentration
                drygrid(ix,jy)=drygrid(ix,jy)*
     +          nclassunc
                drygridtotal=drygridtotal+drygrid(ix,jy)
C Calculate standard deviation of the mean
                drygridsigma(ix,jy)=
     +          drygridsigma(ix,jy)*
     +          sqrt(float(nclassunc))
125             drygridsigmatotal=drygridsigmatotal+
     +          drygridsigma(ix,jy)
            endif

C CONCENTRATION OR MIXING RATIO
            do 120 kz=1,numzgrid
                do 121 l=1,nclassunc
121               auxgrid(l)=gridunc(ix,jy,kz,ks,kp,l,nage)
                call mean(auxgrid,grid(ix,jy,kz),
     +          gridsigma(ix,jy,kz),nclassunc)
C Multiply by number of classes to get total concentration
                grid(ix,jy,kz)=
     +          grid(ix,jy,kz)*nclassunc
                gridtotal=gridtotal+grid(ix,jy,kz)
C Calculate standard deviation of the mean
                gridsigma(ix,jy,kz)=
     +          gridsigma(ix,jy,kz)*
     +          sqrt(float(nclassunc))
                gridsigmatotal=gridsigmatotal+
     +          gridsigma(ix,jy,kz)
120          continue




********************************************************************
C Generate output: may be in concentration (ng/m3) or in mixing
C ratio (ppt) or both
C Output the position and the values alternated multiplied by
C 1 or -1, first line is number of values, number of positions
C For backward simulations, the unit is seconds, stored in grid_time
********************************************************************

C Concentration output
**********************
      if ((iout.eq.1).or.(iout.eq.3).or.(iout.eq.5)) then

C Wet deposition
             sp_count_i=0
             sp_count_r=0
             sp_fact=-1.
             sp_zer=.true.
             if ((ldirect.eq.1).and.(WETDEP)) then
             do 31 jy=0,numygrid-1
                do 31 ix=0,numxgrid-1
concentraion greater zero
                  if (wetgrid(ix,jy).gt.smallnum) then 
                     if (sp_zer.eqv..true.) then ! first non zero value
                        sp_count_i=sp_count_i+1
                        sparse_dump_i(sp_count_i)=ix+jy*numxgrid
                        sp_zer=.false.
                        sp_fact=sp_fact*(-1.)
                     endif
                     sp_count_r=sp_count_r+1
                     sparse_dump_r(sp_count_r)=
     +               sp_fact*1.e12*wetgrid(ix,jy)/area(ix,jy)
c                     sparse_dump_u(sp_count_r)=
c     +                1.e12*wetgridsigma(ix,jy,ks,kp,nage)/area(ix,jy)
                  else ! concentration is zero
                      sp_zer=.true.
                  endif
31           continue
             else
                sp_count_i=0
                sp_count_r=0
             endif
             write(unitoutgrid) sp_count_i
             write(unitoutgrid) (sparse_dump_i(i),i=1,sp_count_i)
             write(unitoutgrid) sp_count_r
             write(unitoutgrid) (sparse_dump_r(i),i=1,sp_count_r)
c            write(unitoutgrid) sp_count_u
c            write(unitoutgrid) (sparse_dump_u(i),i=1,sp_count_r)

C Dry deposition
             sp_count_i=0
             sp_count_r=0
             sp_fact=-1.
             sp_zer=.true.
             if ((ldirect.eq.1).and.(DRYDEP)) then
              do 33 jy=0,numygrid-1
                do 33 ix=0,numxgrid-1
                  if (drygrid(ix,jy).gt.smallnum) then 
                     if (sp_zer.eqv..true.) then ! first non zero value
                        sp_count_i=sp_count_i+1
                        sparse_dump_i(sp_count_i)=ix+jy*numxgrid
                        sp_zer=.false.
                        sp_fact=sp_fact*(-1.)
                     endif
                     sp_count_r=sp_count_r+1
                     sparse_dump_r(sp_count_r)=
     +               sp_fact*
     +                1.e12*drygrid(ix,jy)/area(ix,jy)
c                     sparse_dump_u(sp_count_r)=
c     +                1.e12*drygridsigma(ix,jy,ks,kp,nage)/area(ix,jy)
                  else ! concentration is zero
                      sp_zer=.true.
                  endif
33            continue
             else
                sp_count_i=0
                sp_count_r=0
             endif
             write(unitoutgrid) sp_count_i
             write(unitoutgrid) (sparse_dump_i(i),i=1,sp_count_i)
             write(unitoutgrid) sp_count_r
             write(unitoutgrid) (sparse_dump_r(i),i=1,sp_count_r)
c            write(*,*) sp_count_u
c            write(unitoutgrid) (sparse_dump_u(i),i=1,sp_count_r)



C Concentrations
             sp_count_i=0
             sp_count_r=0
             sp_fact=-1.
             sp_zer=.true.
              do 35 kz=1,numzgrid
                do 35 jy=0,numygrid-1
                  do 35 ix=0,numxgrid-1
                    if (grid(ix,jy,kz).gt.smallnum) then 
                      if (sp_zer.eqv..true.) then ! first non zero value
                        sp_count_i=sp_count_i+1
                        sparse_dump_i(sp_count_i)=
     +                  ix+jy*numxgrid+kz*numxgrid*numygrid
                        sp_zer=.false.
                        sp_fact=sp_fact*(-1.)
                       endif
                       sp_count_r=sp_count_r+1
                       sparse_dump_r(sp_count_r)=
     +                 sp_fact*
     +                  grid(ix,jy,kz)*
     +                  factor3d(ix,jy,kz)/tot_mu(ks,kp)
c                      if ((factor(ix,jy,kz)/tot_mu(ks,kp)).eq.0)
c    +              write (*,*) factor(ix,jy,kz),tot_mu(ks,kp),ks,kp
c                     sparse_dump_u(sp_count_r)=
c     +               ,gridsigma(ix,jy,kz,ks,kp,nage)*
c     +               factor(ix,jy,kz)/tot_mu(ks,kp)
                  else ! concentration is zero
                      sp_zer=.true.
                  endif          
35            continue
             write(unitoutgrid) sp_count_i
             write(unitoutgrid) (sparse_dump_i(i),i=1,sp_count_i)
             write(unitoutgrid) sp_count_r
             write(unitoutgrid) (sparse_dump_r(i),i=1,sp_count_r)
c            write(unitoutgrid) sp_count_u
c            write(unitoutgrid) (sparse_dump_u(i),i=1,sp_count_r)



        endif !  concentration output

C Mixing ratio output
*********************

      if ((iout.eq.2).or.(iout.eq.3)) then      ! mixing ratio

C Wet deposition
             sp_count_i=0
             sp_count_r=0
             sp_fact=-1.
             sp_zer=.true.
             if ((ldirect.eq.1).and.(WETDEP)) then
              do 131 jy=0,numygrid-1
                do 131 ix=0,numxgrid-1
                    if (wetgrid(ix,jy).gt.smallnum) then 
                      if (sp_zer.eqv..true.) then ! first non zero value
                        sp_count_i=sp_count_i+1
                        sparse_dump_i(sp_count_i)=
     +                  ix+jy*numxgrid
                        sp_zer=.false.
                        sp_fact=sp_fact*(-1.)
                     endif
                     sp_count_r=sp_count_r+1
                     sparse_dump_r(sp_count_r)=
     +               sp_fact*
     +               1.e12*wetgrid(ix,jy)/area(ix,jy)
c                     sparse_dump_u(sp_count_r)=
c    +            ,1.e12*wetgridsigma(ix,jy,ks,kp,nage)/area(ix,jy)
                  else ! concentration is zero
                      sp_zer=.true.
                  endif          
131            continue
             else
               sp_count_i=0
               sp_count_r=0
             endif
             write(unitoutgridppt) sp_count_i
             write(unitoutgridppt) (sparse_dump_i(i),i=1,sp_count_i)
             write(unitoutgridppt) sp_count_r
             write(unitoutgridppt) (sparse_dump_r(i),i=1,sp_count_r)
c            write(unitoutgridppt) sp_count_u
c            write(unitoutgridppt) (sparse_dump_u(i),i=1,sp_count_r)


C Dry deposition
             sp_count_i=0
             sp_count_r=0
             sp_fact=-1.
             sp_zer=.true.
             if ((ldirect.eq.1).and.(DRYDEP)) then
              do 133 jy=0,numygrid-1
                do 133 ix=0,numxgrid-1
                    if (drygrid(ix,jy).gt.smallnum) then 
                      if (sp_zer.eqv..true.) then ! first non zero value
                        sp_count_i=sp_count_i+1
                        sparse_dump_i(sp_count_i)=
     +                  ix+jy*numxgrid
                        sp_zer=.false.
                        sp_fact=sp_fact*(-1)
                     endif
                     sp_count_r=sp_count_r+1
                     sparse_dump_r(sp_count_r)=
     +               sp_fact*
     +               1.e12*drygrid(ix,jy)/area(ix,jy)
c                     sparse_dump_u(sp_count_r)=
c    +            ,1.e12*drygridsigma(ix,jy,ks,kp,nage)/area(ix,jy)
                  else ! concentration is zero
                      sp_zer=.true.
                  endif          
133            continue
             else
               sp_count_i=0
               sp_count_r=0
             endif
             write(unitoutgridppt) sp_count_i
             write(unitoutgridppt) (sparse_dump_i(i),i=1,sp_count_i)
             write(unitoutgridppt) sp_count_r
             write(unitoutgridppt) (sparse_dump_r(i),i=1,sp_count_r)
c            write(unitoutgridppt) sp_count_u
c            write(unitoutgridppt) (sparse_dump_u(i),i=1,sp_count_r)


C Mixing ratios
             sp_count_i=0
             sp_count_r=0
             sp_fact=-1.
             sp_zer=.true.
              do 135 kz=1,numzgrid
                do 135 jy=0,numygrid-1
                  do 135 ix=0,numxgrid-1
                    if (grid(ix,jy,kz).gt.smallnum) then 
                      if (sp_zer.eqv..true.) then ! first non zero value
                        sp_count_i=sp_count_i+1
                        sparse_dump_i(sp_count_i)=
     +                  ix+jy*numxgrid+kz*numxgrid*numygrid
                        sp_zer=.false.
                        sp_fact=sp_fact*(-1.)
                     endif
                     sp_count_r=sp_count_r+1
                     sparse_dump_r(sp_count_r)=
     +               sp_fact*
     +                1.e12*grid(ix,jy,kz)
     +                /volume(ix,jy,kz)/outnum*
     +                weightair/weightmolar(ks)/densityoutgrid(ix,jy,kz) 
c                     sparse_dump_u(sp_count_r)=
c     +              ,1.e12*gridsigma(ix,jy,kz,ks,kp,nage)/volume(ix,jy,kz)/
c     +              outnum*weightair/weightmolar(ks)/
c     +              densityoutgrid(ix,jy,kz)
                  else ! concentration is zero
                      sp_zer=.true.
                  endif          
135            continue
             write(unitoutgridppt) sp_count_i
             write(unitoutgridppt) (sparse_dump_i(i),i=1,sp_count_i)
             write(unitoutgridppt) sp_count_r
             write(unitoutgridppt) (sparse_dump_r(i),i=1,sp_count_r)
c            write(unitoutgridppt) sp_count_u
c            write(unitoutgridppt) (sparse_dump_u(i),i=1,sp_count_r)

          endif ! output for ppt

12      continue ! maxpointspec and ageclass

        close(unitoutgridppt)
        close(unitoutgrid)

2       continue ! species

      if (gridtotal.gt.0.) gridtotalunc=gridsigmatotal/gridtotal
      if (wetgridtotal.gt.0.) wetgridtotalunc=wetgridsigmatotal/
     +wetgridtotal
      if (drygridtotal.gt.0.) drygridtotalunc=drygridsigmatotal/
     +drygridtotal

C Dump of receptor concentrations

        if (numreceptor.gt.0) then
          write(unitoutreceptppt) itime
          do 150 ks=1,nspec
150         write(unitoutreceptppt) (1.e12*creceptor(i,ks)/outnum*
     +    weightair/weightmolar(ks)/densityoutrecept(i),i=1,numreceptor)
        endif

C Dump of receptor concentrations

        if (numreceptor.gt.0) then
          write(unitoutrecept) itime
          do 50 ks=1,nspec
50          write(unitoutrecept) (1.e12*creceptor(i,ks)/outnum,
     +      i=1,numreceptor)
        endif



C Reinitialization of grid
**************************

      do 40 ks=1,nspec
      do 40 kp=1,maxpointspec_act
        do 42 i=1,numreceptor
42        creceptor(i,ks)=0.
        do 40 jy=0,numygrid-1
          do 40 ix=0,numxgrid-1
            do 40 l=1,nclassunc
              do 40 nage=1,nageclass
                do 40 kz=1,numzgrid
40                gridunc(ix,jy,kz,ks,kp,l,nage)=0.


      end
