***********************************************************************
* Copyright 1998,1999,2000,2001,2002,2005,2007,2008,2009,2010         *
* Andreas Stohl, Petra Seibert, A. Frank, Gerhard Wotawa,             *
* Caroline Forster, Sabine Eckhardt, John Burkhart, Harald Sodemann   *
*                                                                     *
* This file is part of FLEXPART.                                      *
*                                                                     *
* FLEXPART is free software: you can redistribute it and/or modify    *
* it under the terms of the GNU General Public License as published by*
* the Free Software Foundation, either version 3 of the License, or   *
* (at your option) any later version.                                 *
*                                                                     *
* FLEXPART is distributed in the hope that it will be useful,         *
* but WITHOUT ANY WARRANTY; without even the implied warranty of      *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
* GNU General Public License for more details.                        *
*                                                                     *
* You should have received a copy of the GNU General Public License   *
* along with FLEXPART.  If not, see <http://www.gnu.org/licenses/>.   *
***********************************************************************

C-----------------------------------------------------------------------
      FUNCTION DISTANCE2(RLAT1,RLON1,RLAT2,RLON2)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C
C SUBPROGRAM:  GCDIST     COMPUTE GREAT CIRCLE DISTANCE
C   PRGMMR: IREDELL       ORG: W/NMC23       DATE: 96-04-10
C
C ABSTRACT: THIS SUBPROGRAM COMPUTES GREAT CIRCLE DISTANCE
C           BETWEEN TWO POINTS ON THE EARTH. COORDINATES ARE GIVEN IN RADIANS!
C
C PROGRAM HISTORY LOG:
C   96-04-10  IREDELL
C
C USAGE:    ...GCDIST(RLAT1,RLON1,RLAT2,RLON2)
C
C   INPUT ARGUMENT LIST:
C     RLAT1    - REAL LATITUDE OF POINT 1 IN RADIANS
C     RLON1    - REAL LONGITUDE OF POINT 1 IN RADIANS
C     RLAT2    - REAL LATITUDE OF POINT 2 IN RADIANS
C     RLON2    - REAL LONGITUDE OF POINT 2 IN RADIANS
C
C   OUTPUT ARGUMENT LIST:
C     GCDIST   - REAL GREAT CIRCLE DISTANCE IN KM
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C
C$$$

      REAL RLAT1,RLON1,RLAT2,RLON2
      DOUBLE PRECISION RERTH,PI,CLAT1,CLAT2,SLAT2,CDLON,CRD
      PARAMETER(RERTH=6.3712E6)
      PARAMETER(PI=3.14159265358979)

      IF ((ABS(RLAT1-RLAT2).LT.0.0003).AND.
     +    (ABS(RLON1-RLON2).LT.0.0003)) THEN
        DISTANCE=0.
      ELSE

C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        CLAT1=DCOS(DBLE(RLAT1))
        SLAT1=DSIN(DBLE(RLAT1))
        CLAT2=DCOS(DBLE(RLAT2))
        SLAT2=DSIN(DBLE(RLAT2))
        CDLON=DCOS(DBLE(RLON1-RLON2))
        CRD=SLAT1*SLAT2+CLAT1*CLAT2*CDLON
        DISTANCE2=SNGL(RERTH*DACOS(CRD)/1000.)
      ENDIF
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      END
