***********************************************************************
* Copyright 1998,1999,2000,2001,2002,2005,2007,2008,2009,2010         *
* Andreas Stohl, Petra Seibert, A. Frank, Gerhard Wotawa,             *
* Caroline Forster, Sabine Eckhardt, John Burkhart, Harald Sodemann   *
*                                                                     *
* This file is part of FLEXPART.                                      *
*                                                                     *
* FLEXPART is free software: you can redistribute it and/or modify    *
* it under the terms of the GNU General Public License as published by*
* the Free Software Foundation, either version 3 of the License, or   *
* (at your option) any later version.                                 *
*                                                                     *
* FLEXPART is distributed in the hope that it will be useful,         *
* but WITHOUT ANY WARRANTY; without even the implied warranty of      *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
* GNU General Public License for more details.                        *
*                                                                     *
* You should have received a copy of the GNU General Public License   *
* along with FLEXPART.  If not, see <http://www.gnu.org/licenses/>.   *
***********************************************************************

      subroutine initial_cond_calc(itime,i)
C                                    i   i
********************************************************************************
*                                                                              *
*     Calculation of the sensitivity to initial conditions for BW runs         *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     15 January 2010                                                          *
*                                                                              *
********************************************************************************

      use unc_mod
      use outg_mod

      include 'includepar'
      include 'includecom'

      integer itime,i,ix,jy,ixp,jyp,kz,ks
      integer il,ind,indz,indzp,nrelpointer
      real rddx,rddy,p1,p2,p3,p4,dz1,dz2,dz
      real ddx,ddy
      real rhoprof(2),rhoi,xl,yl,wx,wy,w


C For forward simulations, make a loop over the number of species;
C for backward simulations, make an additional loop over the release points
***************************************************************************


      if (itra1(i).ne.itime) return

C Depending on output option, calculate air density or set it to 1
C linit_cond: 1=mass unit, 2=mass mixing ratio unit
******************************************************************


      if (linit_cond.eq.1) then     ! mass unit

        ix=int(xtra1(i))
        jy=int(ytra1(i))
        ixp=ix+1
        jyp=jy+1
        ddx=xtra1(i)-float(ix)
        ddy=ytra1(i)-float(jy)
        rddx=1.-ddx
        rddy=1.-ddy
        p1=rddx*rddy
        p2=ddx*rddy
        p3=rddx*ddy
        p4=ddx*ddy

        do 5 il=2,nz
          if (height(il).gt.ztra1(i)) then
            indz=il-1
            indzp=il
            goto 6
          endif
5         continue
6       continue

        dz1=ztra1(i)-height(indz)
        dz2=height(indzp)-ztra1(i)
        dz=1./(dz1+dz2)

C Take density from 2nd wind field in memory (accurate enough, no time interpolation needed)
********************************************************************************************
        do 70 ind=indz,indzp
70        rhoprof(ind-indz+1)=p1*rho(ix ,jy ,ind,2)
     +                       +p2*rho(ixp,jy ,ind,2)
     +                       +p3*rho(ix ,jyp,ind,2)
     +                       +p4*rho(ixp,jyp,ind,2)
        rhoi=(dz1*rhoprof(2)+dz2*rhoprof(1))*dz
      elseif (linit_cond.eq.2) then    ! mass mixing ratio unit
        rhoi=1.
      endif


*****************************************************************************
C 1. Evaluate grid concentrations using a uniform kernel of bandwidths dx, dy
*****************************************************************************


C For backward simulations, look from which release point the particle comes from
C For domain-filling trajectory option, npoint contains a consecutive particle
C number, not the release point information. Therefore, nrelpointer is set to 1
C for the domain-filling option.
*********************************************************************************

      if ((ioutputforeachrelease.eq.0).or.(mdomainfill.eq.1)) then
        nrelpointer=1
      else
        nrelpointer=npoint(i)
      endif
        
      do 30 kz=1,numzgrid                ! determine height of cell
        if (outheight(kz).gt.ztra1(i)) goto 21
30      continue
21    continue
      if (kz.le.numzgrid) then           ! inside output domain


        xl=(xtra1(i)*dx+xoutshift)/dxout
        yl=(ytra1(i)*dy+youtshift)/dyout
        ix=int(xl)
        if (xl.lt.0.) ix=ix-1
        jy=int(yl)
        if (yl.lt.0.) jy=jy-1


C If a particle is close to the domain boundary, do not use the kernel either
*****************************************************************************

        if ((xl.lt.0.5).or.(yl.lt.0.5).or.
     +      (xl.gt.float(numxgrid-1)-0.5).or.
     +      (yl.gt.float(numygrid-1)-0.5)) then             ! no kernel, direct attribution to grid cell
          if ((ix.ge.0).and.(jy.ge.0).and.(ix.le.numxgrid-1).and.
     +    (jy.le.numygrid-1)) then
            do 42 ks=1,nspec
42            init_cond(ix,jy,kz,ks,nrelpointer)=
     +        init_cond(ix,jy,kz,ks,nrelpointer)+
     +        xmass1(i,ks)/rhoi
          endif

        else                                 ! attribution via uniform kernel

          ddx=xl-float(ix)                   ! distance to left cell border
          ddy=yl-float(jy)                   ! distance to lower cell border
          if (ddx.gt.0.5) then
            ixp=ix+1
            wx=1.5-ddx
          else
            ixp=ix-1
            wx=0.5+ddx
          endif

          if (ddy.gt.0.5) then
            jyp=jy+1
            wy=1.5-ddy
          else
            jyp=jy-1
            wy=0.5+ddy
          endif


C Determine mass fractions for four grid points
***********************************************

          if ((ix.ge.0).and.(ix.le.numxgrid-1)) then
            if ((jy.ge.0).and.(jy.le.numygrid-1)) then
              w=wx*wy
              do 22 ks=1,nspec
22              init_cond(ix,jy,kz,ks,nrelpointer)=
     +          init_cond(ix,jy,kz,ks,nrelpointer)+xmass1(i,ks)/rhoi*w
            endif

            if ((jyp.ge.0).and.(jyp.le.numygrid-1)) then
              w=wx*(1.-wy)
              do 25 ks=1,nspec
25              init_cond(ix,jyp,kz,ks,nrelpointer)=
     +          init_cond(ix,jyp,kz,ks,nrelpointer)+xmass1(i,ks)/rhoi*w
            endif
          endif


          if ((ixp.ge.0).and.(ixp.le.numxgrid-1)) then
            if ((jyp.ge.0).and.(jyp.le.numygrid-1)) then
              w=(1.-wx)*(1.-wy)
              do 23 ks=1,nspec
23              init_cond(ixp,jyp,kz,ks,nrelpointer)=
     +          init_cond(ixp,jyp,kz,ks,nrelpointer)+xmass1(i,ks)/rhoi*w
            endif

            if ((jy.ge.0).and.(jy.le.numygrid-1)) then
              w=(1.-wx)*wy
              do 24 ks=1,nspec
24              init_cond(ixp,jy,kz,ks,nrelpointer)=
     +          init_cond(ixp,jy,kz,ks,nrelpointer)+xmass1(i,ks)/rhoi*w
            endif
          endif
        endif

      endif

      end
